use crate::layers::{Layer, LayerError};
use log::info;

#[derive(Clone)]
pub struct LoggerLayer;

impl Layer for LoggerLayer {
    fn frame(&mut self, data: &mut Vec<u8>) -> Result<(), LayerError> {
        info!("Sending: {}", pretty_hex::pretty_hex(data));
        Ok(())
    }

    fn deframe(&mut self, data: &mut Vec<u8>) -> Result<(), LayerError> {
        info!("Received: {}", pretty_hex::pretty_hex(data));
        Ok(())
    }

    fn split(&self) -> (Box<dyn Layer>, Box<dyn Layer>) {
        (Box::new(self.clone()), Box::new(self.clone()))
    }

    fn name(&self) -> String {
        "Logging".to_string()
    }
}
