mod logger_layer;

use crate::layers::logger_layer::LoggerLayer;
use serde::Deserialize;
use thiserror::Error;

#[derive(Debug, Error)]
pub enum LayerError {}

pub trait Layer: Send {
    fn frame(&mut self, data: &mut Vec<u8>) -> Result<(), LayerError>;

    fn deframe(&mut self, data: &mut Vec<u8>) -> Result<(), LayerError>;

    fn split(&self) -> (Box<dyn Layer>, Box<dyn Layer>);

    fn name(&self) -> String;
}

#[derive(Debug, Deserialize)]
#[serde(tag = "layer")]
pub enum Layers {
    Logger,
}

impl Layers {
    pub fn get_layer(&self) -> Box<dyn Layer> {
        match self {
            Layers::Logger => Box::new(LoggerLayer {}),
        }
    }
}
