use crate::layers::{Layer, LayerError};
use thiserror::Error;

pub mod tun_rx_worker;
pub mod tun_tx_worker;

#[derive(Debug, Error)]
pub enum Error {
    #[error("Error with layer processing: {0}")]
    Layer(#[from] LayerError),
    #[error("IO Error: {0}")]
    IO(#[from] std::io::Error),
}

pub trait TunWorker {
    fn worker_thread(self);
    #[allow(dead_code)]
    fn add_layer(&mut self, layer: Box<dyn Layer>);

    fn get_layers(&mut self) -> &mut Vec<Box<dyn Layer>>;

    fn frame_message(&mut self, msg: &mut Vec<u8>) -> Result<(), Error> {
        for layer in self.get_layers() {
            layer.frame(msg)?;
        }

        Ok(())
    }

    fn deframe_message(&mut self, msg: &mut Vec<u8>) -> Result<(), Error> {
        for layer in self.get_layers().iter_mut().rev() {
            layer.deframe(msg)?;
        }

        Ok(())
    }
}
