use crate::layers::Layer;
use crate::medium::ReadMedium;
use crate::tun_worker::TunWorker;
use std::io::Write;

pub struct TunRxWorker {
    layers: Vec<Box<dyn Layer>>,
    tun_writer: tun::Writer,
    rx_medium: Box<dyn ReadMedium>,
    bytes_read: usize,
    bytes_written: usize,
}

impl TunRxWorker {
    pub fn new(tun_writer: tun::Writer, medium: Box<dyn ReadMedium>) -> Self {
        Self {
            layers: vec![],
            tun_writer,
            rx_medium: medium,
            bytes_read: 0,
            bytes_written: 0,
        }
    }
}

impl TunWorker for TunRxWorker {
    fn worker_thread(mut self) {
        let mut buffer = vec![0; 4096];
        loop {
            buffer.resize(4096, 0);
            buffer.fill(0);
            let data_read = self.rx_medium.read(&mut buffer).unwrap();

            buffer.resize(data_read, 0);

            if data_read == 0 {
                log::info!("TUN device closed exiting...");
                return;
            }

            self.bytes_read += data_read;

            if let Err(e) = self.deframe_message(&mut buffer) {
                log::error!("Failed to deframe message: {}", e);
                continue;
            }

            if let Err(e) = self.tun_writer.write_all(&buffer) {
                log::error!("Failed to write message to medium: {}", e)
            }

            self.bytes_written += buffer.len();
        }
    }

    fn add_layer(&mut self, layer: Box<dyn Layer>) {
        self.layers.push(layer)
    }

    fn get_layers(&mut self) -> &mut Vec<Box<dyn Layer>> {
        &mut self.layers
    }
}
